@@ -1,17 +1,21 @@
 BUILD_BIT='32bit'
 BUILD_CPU='ARMv7 Processor rev 3 (v7l)'
-BUILD_DATE='2020-08-10'
+BUILD_DATE='2020-12-23'
 BUILD_MACHINE='armv7l'
 BUILD_OS='Linux unknown unknown unknown'
-BUILD_TIME='12:30'
-COMPILER_VERSION='armv7l-unknown-linux-gnueabihf-gcc (GCC) 9.3.0'
+BUILD_TIME='01:41'
+COMPILER_VERSION='armv7l-unknown-linux-gnueabihf-gcc (GCC) 10.2.0'
 CYTHON_VERSION='0.29.21'
-LINKER_VERSION='GNU ld (GNU Binutils) 2.34'
-PYTHON_VERSION='3.8.5'
+LINKER_VERSION='GNU ld (GNU Binutils) 2.35'
+PYTHON_VERSION='3.9.1'
 RELEASE_BUILD=True
-lib_gtk='3.24.21'
+lib_gtk='3.24.24'
+lib_gtk_x11='3.24.24'
+lib_py3cairo='1.20.0'
+lib_pygobject='3.38.0'
+lib_python3='3.9'
 lib_vpx='1.9.0'
-lib_x11='1.6.10'
+lib_x11='1.7.0'
 lib_x264='0.160.3011'
 lib_x265='3.4'
 lib_xcomposite='0.4.5'
