@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -11,8 +11,8 @@ import os.path
 import hashlib
 
 from xpra.simple_stats import to_std_unit
-from xpra.os_util import bytestostr, WIN32
-from xpra.util import engs, repr_ellipsized
+from xpra.os_util import bytestostr, osexpand, load_binary_file, WIN32, POSIX
+from xpra.util import engs, repr_ellipsized, XPRA_FILETRANSFER_NOTIFICATION_ID
 from xpra.net.file_transfer import FileTransferAttributes
 from xpra.server.mixins.stub_server_mixin import StubServerMixin
 from xpra.log import Logger
@@ -49,7 +49,7 @@ class FilePrintServer(StubServerMixin):
 
     def init_sockets(self, sockets):
         #verify we have a local socket for printing:
-        unixsockets = [info for socktype, _, info in sockets if socktype=="unix-domain"]
+        unixsockets = [info for socktype, _, info, _ in sockets if socktype=="unix-domain"]
         printlog("local unix domain sockets we can use for printing: %s", unixsockets)
         if not unixsockets and self.file_transfer.printing:
             if not WIN32:
@@ -62,21 +62,29 @@ class FilePrintServer(StubServerMixin):
     def get_server_features(self, _source):
         f = self.file_transfer.get_file_transfer_features()
         f["printer.attributes"] = ("printer-info", "device-uri")
-        f.update(self.file_transfer.get_file_transfer_features())
+        ftf = self.file_transfer.get_file_transfer_features()
+        if self.file_transfer.file_transfer:
+            ftf["request-file"] = True
+        f.update(ftf)
         return f
 
     def get_info(self, _proto):
-        d = {
-            "lpadmin"              : self.lpadmin,
-            "lpinfo"               : self.lpinfo,
-            "add-printer-options"  : self.add_printer_options,
-            }
+        d = {}
+        if POSIX:
+            d.update({
+                "lpadmin"              : self.lpadmin,
+                "lpinfo"               : self.lpinfo,
+                "add-printer-options"  : self.add_printer_options,
+                })
         if self.file_transfer.printing:
             from xpra.platform.printing import get_info
             d.update(get_info())
         info = {"printing" : d}
         if self.file_transfer.file_transfer:
-            info["file"] = self.file_transfer.get_info()
+            fti = self.file_transfer.get_info()
+            if self.file_transfer.file_transfer:
+                fti["request-file"] = True
+            info["file"] = fti
         return info
 
 
@@ -133,16 +141,22 @@ class FilePrintServer(StubServerMixin):
             printlog.error("Error: invalid print packet, only %i arguments", len(packet))
             printlog.error(" %s", [repr_ellipsized(x) for x in packet])
             return
-        filename, file_data = packet[1:3]
+        def s(b):
+            try:
+                return b.decode("utf-8")
+            except Exception:
+                return bytestostr(b)
+        filename = s(packet[1])
+        file_data = packet[2]
         mimetype, source_uuid, title, printer, no_copies, print_options = "", "*", "unnamed document", "", 1, ""
         if len(packet)>=4:
-            mimetype = packet[3]
+            mimetype = bytestostr(packet[3])
         if len(packet)>=5:
             source_uuid = bytestostr(packet[4])
         if len(packet)>=6:
-            title = packet[5]
+            title = s(packet[5])
         if len(packet)>=7:
-            printer = packet[6]
+            printer = bytestostr(packet[6])
         if len(packet)>=8:
             no_copies = int(packet[7])
         if len(packet)>=9:
@@ -223,7 +237,7 @@ class FilePrintServer(StubServerMixin):
             printlog.error("Error: received printer definitions data")
             printlog.error(" but this server does not support printer forwarding")
             return
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if ss is None:
             return
         printers = packet[1]
@@ -234,52 +248,88 @@ class FilePrintServer(StubServerMixin):
     ######################################################################
     # file transfers:
     def _process_send_file(self, proto, packet):
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if not ss:
             printlog.warn("Warning: invalid client source for send-file packet")
             return
         ss._process_send_file(packet)
 
     def _process_ack_file_chunk(self, proto, packet):
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if not ss:
             printlog.warn("Warning: invalid client source for ack-file-chunk packet")
             return
         ss._process_ack_file_chunk(packet)
 
     def _process_send_file_chunk(self, proto, packet):
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if not ss:
             printlog.warn("Warning: invalid client source for send-file-chunk packet")
             return
         ss._process_send_file_chunk(packet)
 
     def _process_send_data_request(self, proto, packet):
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if not ss:
             printlog.warn("Warning: invalid client source for send-file-request packet")
             return
         ss._process_send_data_request(packet)
 
     def _process_send_data_response(self, proto, packet):
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if not ss:
             printlog.warn("Warning: invalid client source for send-data-response packet")
             return
         ss._process_send_data_response(packet)
 
+    def _process_request_file(self, proto, packet):
+        ss = self.get_server_source(proto)
+        if not ss:
+            printlog.warn("Warning: invalid client source for send-data-response packet")
+            return
+        try:
+            argf = packet[1].decode("utf-8")
+        except UnicodeDecodeError:
+            argf = bytestostr(packet[1])
+        openit = packet[2]
+        filename = os.path.abspath(osexpand(argf))
+        if not os.path.exists(filename):
+            filelog.warn("Warning: the file requested does not exist:")
+            filelog.warn(" %s", filename)
+            ss.may_notify(XPRA_FILETRANSFER_NOTIFICATION_ID,
+                          "File not found", "The file requested does not exist:\n%s" % filename,
+                           icon_name="file")
+            return
+        try:
+            stat = os.stat(filename)
+            filelog("os.stat(%s)=%s", filename, stat)
+        except os.error:
+            filelog("os.stat(%s)", filename, exc_info=True)
+        else:
+            file_size_MB = stat.st_size//1024//1024
+            if file_size_MB>self.file_transfer.file_size_limit or file_size_MB>ss.file_size_limit:
+                ss.may_notify(XPRA_FILETRANSFER_NOTIFICATION_ID,
+                              "File too large",
+                              "The file requested is too large to send:\n%s\nis %iMB" % (argf, file_size_MB),
+                               icon_name="file")
+                return
+        data = load_binary_file(filename)
+        ss.send_file(filename, "", data, len(data), openit=openit)
+
 
     def init_packet_handlers(self):
         if self.file_transfer.printing:
-            self._authenticated_packet_handlers.update({
+            self.add_packet_handlers({
                 "printers":                             self._process_printers,
                 "print":                                self._process_print,
-              })
+              }, False)
         if self.file_transfer.printing or self.file_transfer.file_transfer:
-            self._authenticated_packet_handlers.update({
+            self.add_packet_handlers({
                 "send-file":                            self._process_send_file,
                 "ack-file-chunk":                       self._process_ack_file_chunk,
                 "send-file-chunk":                      self._process_send_file_chunk,
                 "send-data-request":                    self._process_send_data_request,
                 "send-data-response":                   self._process_send_data_response,
-              })
+              }, False)
+        if self.file_transfer.file_transfer:
+            self.add_packet_handler("request-file",     self._process_request_file, False)
